/*********************************************************************************
 SerialTx
 CFunction to create additional Tx only serial outputs for the Micromite
 Geoff Graham, Sept 2014

 usage:  r = SerialTx(port, baud-rate, string)
 where   port               is the output pin in port A (see below for an explanation)
         baud-rate          is the desired transmit speed (maximum 38400 at 40MHz)
         string             is the string of characters to send
         r                  is the return value which is always zero

 port can be one of the following:
         port  |  I/O pin on a   |  I/O pin on a
         nbr   |  28-pin chip    |  44-pin chip
         ======|=================|=============
          0            2                19
          1            3                20
          2            9                30
          3           10                31
          7                             13
          8                             32
          9                             35
         10                             12

 *************************************************************************************/

#include <plib.h>

void Delay(unsigned int);
unsigned int GetCurrentCpuSpeed(void);

long long int main(unsigned int *port, unsigned int *BaudRate, char *string) {
    unsigned int ticks = (GetCurrentCpuSpeed()/(*BaudRate * 2))-5;  // calculate the number of core timer ticks for each bit
    unsigned char mask;
    int count = 0;

    while(count++ < string[0]) {
        LATACLR = 1 << *port;                                       // send the startbit
        Delay(ticks);                                               // wait for the specified baud rate timing
        for (mask = 0x01; mask > 0; mask <<= 1) {
            if(string[count] & mask)                                // check the bit to send
                LATASET = 1 << *port;                               // send 1
            else
                LATACLR = 1 << *port;                               // send 0
            Delay(ticks);                                           // wait for the specified baud rate timing
      }
      LATASET = 1 << *port;                                         // send the stop bit
      Delay(ticks);                                                 // wait for the specified baud rate timing
    }
    return 0;
}


// delay for a certain number of core timer ticks (one tick every 50nS at 40MHz CPU speed, 100nS at 20MHz, etc)
void Delay(unsigned int ticks) {
    unsigned int current_ticks = 0;
     asm volatile("mtc0   %0, $9": "+r"(current_ticks));            // set the core timer to zero
     do {
         asm volatile("mfc0   %0, $9" : "=r"(current_ticks));       // get the core timer ticks
     } while(current_ticks < ticks);
}


// function to figure out what speed the CPU is running at (required to calculate delays)
unsigned int GetCurrentCpuSpeed(void) {
    unsigned int i;
    i = (4000000 * (OSCCONbits.PLLMULT + 15)) >> OSCCONbits.PLLODIV;
    if(i == 44000000) i = 48000000;                                 // 48MHz is a special case
    return i;
}
